import net.jxta.platform.NetworkManager;
import net.jxta.platform.NetworkConfigurator;

import net.jxta.peergroup.PeerGroup;

import net.jxta.exception.PeerGroupException;

import net.jxta.discovery.DiscoveryService;
import net.jxta.discovery.DiscoveryEvent;
import net.jxta.discovery.DiscoveryListener;

import java.text.MessageFormat;
import java.io.File;

public class HelloWorld implements DiscoveryListener{
	/* attributes */
	NetworkManager manager;
	NetworkConfigurator theNetworkConfigurator;
	PeerGroup thePeerGroup;
	DiscoveryService discovery;
	
	boolean receivedDiscoveryEventReply;

	/*constructors*/
	HelloWorld(){
		/* initialize the network manager */
		try{
		
			/* can connect to a rendezvous peer only if configured correctly.
			   for the moment there seem to be no active public rendezvous peers */
			/* manager = new NetworkManager( NetworkManager.ConfigMode.EDGE, "HelloWorld" ); */ 
			manager = new NetworkManager( NetworkManager.ConfigMode.ADHOC, "HelloWorld", new File( new File(".cache"), "HelloWorld" ).toURI() );  
			manager.setConfigPersistent(true);
			theNetworkConfigurator = manager.getConfigurator();
			
			if (theNetworkConfigurator.exists()){
				System.out.println("Configuration found ...");
				File networkConfig = new File ( theNetworkConfigurator.getHome(), "PlatformConfig" );
				try{
					theNetworkConfigurator.load( networkConfig.toURI());
				}catch(Exception e){
					e.printStackTrace();
					System.exit(-1);
				}
			}else{
				System.out.println("Configuration not found. Creating new one !");
				theNetworkConfigurator.setName( "HelloWorld_n");
				theNetworkConfigurator.setPrincipal ("myLogin");
				theNetworkConfigurator.setPassword ("12345678");
				theNetworkConfigurator.save();
			}
			
			System.out.println("Starting JXTA");
			thePeerGroup = manager.startNetwork();
			System.out.println("JXTA Started");
		}catch(Exception e){
			e.printStackTrace();
			System.exit(-1);
		}
		
		receivedDiscoveryEventReply = false;
	}
	
	/*statics */
	public static void main(String args[]){
		HelloWorld myHelloWorld = new HelloWorld(); /* init the JXTA network */
		myHelloWorld.getInfo();
		myHelloWorld.connectToRendezvous();
		myHelloWorld.initDiscoveyService();
		myHelloWorld.sendDiscoveryRequests();
		myHelloWorld.stopJXTA();	
	}

	/*methods*/
	void getInfo(){
		System.out.println ( "Peer name       :" + thePeerGroup.getPeerName() );
		System.out.println ( "Peer ID         :" + thePeerGroup.getPeerID().toString() );
		System.out.println ( "Peer group name :" + thePeerGroup.getPeerGroupName() );
		System.out.println ( "Peer group ID   :" + thePeerGroup.getPeerGroupID().toString() );

	}
	
	void connectToRendezvous(){
		System.out.println("Waiting for a rendezvous connection");
		boolean connected = manager.waitForRendezvousConnection(12000); /*wait for 12 seconds */
		System.out.println( MessageFormat.format("Connected : {0}", connected ));
	}
	
	void initDiscoveyService(){
		discovery = thePeerGroup.getDiscoveryService();
		discovery.addDiscoveryListener(this);
	}
	
	public void discoveryEvent(DiscoveryEvent ev){
		System.out.println("Discovery event ..... your code here");
		receivedDiscoveryEventReply = true;
		
	}
	
	void sendDiscoveryRequests(){
		while ( ! receivedDiscoveryEventReply ){
			/* send disovery request message */
			System.out.println("Sending discovery message");
			discovery.getRemoteAdvertisements(
				null,                 //not sent to a specific peer
				DiscoveryService.ADV, //advertisment type
				null,                 //any type of advertisment
				null,                 //any value
				1,                    //one discovery respones is all we are looking for
				null                  //using global listener
			);
			
			try{
				Thread.sleep(60000);
			}catch(Exception e){
				e.printStackTrace();
				System.exit(-1);
			}
		}	
	}

	void stopJXTA(){
		manager.stopNetwork();
	}

}
